//CN-Palm Summer 2003

#include "win.h"

static void move_up(void);
static void draw_char(UInt8 x, Int16 chr);
static void append_chars(const Char *msg, UInt16 len);

extern UInt32 romVersion;
MemHandle fntH;

static void move_up()
{
	//Move messages up
	RectangleType rect={{0,fontHeight},{bufferWidth,bufferInsY}};
	WinCopyRectangle(bufferH, bufferH, &rect, 0, 0, winPaint);

	//Clear bottom line
	rect.topLeft.y = bufferInsY;
	rect.extent.y  = fontHeight;

	WinEraseRectangle(&rect, 0);
}

static void draw_char(UInt8 x, Int16 chr)
{
	Char str[8];
/*
	if ((chr < 32 || chr > 127) && chr != 223) //' ', '~', ''
		return;
*/
	StrPrintF(str, "%c", chr);
	WinDrawChars(str, 1, x,bufferInsY);
}

static void append_chars(const Char *msg, UInt16 len)
{
	UInt16 i, x=0;
	FontID oldFnt;
	oldFnt = FntSetFont(128);
	WinSetDrawWindow(bufferH);
	move_up();

	for(i=0; i<len; i++)
	{
		draw_char(x, msg[i]);
		x = x + fontWidth;
		if (x >= bufferWidth)
		{
			if (i+1 >= len) break;
			move_up();
			x = fontWidth;
			if (msg[i+1] == ' ') i++;
		}
	}

	WinSetDrawWindow(screenH);
	FntSetFont(oldFnt);
}

void WRefresh()
{
	//Clear screen
	RectangleType rect={{0,16},{bufferWidth,bufferHeight}};
	WinEraseRectangle(&rect, 0);

	//Re-Draw messages
	rect.topLeft.y = 0;
	WinCopyRectangle(bufferH,screenH, &rect, 0,16, winPaint);
}

void WAddMessage(IndexedColorType color, const Char *msg) //UInt8
{
	UInt16 len = StrLen(msg);
	if (len > 160) len = 160;

	//Insert new message
	if (romVersion >= sysMakeROMVersion(3,5,0,sysROMStageRelease,0))
	{
		WinPushDrawState();
		WinSetBackColor(color);
	}
	append_chars(msg, len);
	if (romVersion >= sysMakeROMVersion(3,5,0,sysROMStageRelease,0))
	{
		WinPopDrawState();
	}

	//Re-Draw messages
	WRefresh();
}

void WInit()
{
	UInt16 error;
	RectangleType rect={{0,0},{bufferWidth,bufferHeight}};
	FontPtr fntPtr;

	//Get current buffer
	screenH = WinGetDrawWindow();

	//Create offscreen buffer
	bufferH = WinCreateOffscreenWindow(bufferWidth, bufferHeight, 
		screenFormat, &error);
	WinSetDrawWindow(bufferH);
	WinEraseRectangle(&rect, 0); //Clear for use
	WinSetDrawWindow(screenH);

	//Load font
	fntH  = DmGetResource(fontRscType, 1000);
	fntPtr = (FontPtr)MemHandleLock(fntH);
	FntDefineFont(128, fntPtr);
	MemHandleUnlock(fntH);

	//PilRC example doesn't release the resource
	//DmReleaseResource(fntH);
}

void WEnd()
{
	//Delete the off screen buffer
	WinDeleteWindow(bufferH, false);
}
